function gogma_align(gmmx::AbstractGMM, gmmy::AbstractGMM; interactions=nothing, kwargs...)
    pσ, pϕ = pairwise_consts(gmmx,gmmy,interactions)
    boundsfun(x,y,block) = gauss_l2_bounds(x,y,block,pσ,pϕ)
    localfun(x,y,block) = local_align(x,y,block,pσ,pϕ)
    return branchbound(gmmx, gmmy; boundsfun=boundsfun, localfun=localfun, kwargs...)
end
function rot_gogma_align(gmmx::AbstractGMM, gmmy::AbstractGMM; interactions=nothing, kwargs...)
    pσ, pϕ = pairwise_consts(gmmx,gmmy,interactions)
    boundsfun(x,y,block) = gauss_l2_bounds(x,y,block,pσ,pϕ)
    localfun(x,y,block) = local_align(x,y,block,pσ,pϕ)
    rot_branchbound(gmmx, gmmy; boundsfun=boundsfun, localfun=localfun, kwargs...)
end
function trl_gogma_align(gmmx::AbstractGMM, gmmy::AbstractGMM; interactions=nothing, kwargs...) 
    pσ, pϕ = pairwise_consts(gmmx,gmmy,interactions)
    boundsfun(x,y,block) = gauss_l2_bounds(x,y,block,pσ,pϕ)
    localfun(x,y,block) = local_align(x,y,block,pσ,pϕ)
    trl_branchbound(gmmx, gmmy; boundsfun=boundsfun, localfun=localfun, kwargs...)
end
function tiv_gogma_align(gmmx::AbstractGMM, gmmy::AbstractGMM, cx=Inf, cy=Inf; kwargs...)
    tivgmmx, tivgmmy = tivgmm(gmmx, cx), tivgmm(gmmy, cy)
    pσ, pϕ = pairwise_consts(gmmx,gmmy)
    tivpσ, tivpϕ = pairwise_consts(tivgmmx,tivgmmy)
    boundsfun(x,y,block) = gauss_l2_bounds(x,y,block,pσ,pϕ)
    rot_boundsfun(x,y,block) = gauss_l2_bounds(x,y,block,tivpσ,tivpϕ)
    localfun(x,y,block) = local_align(x,y,block,pσ,pϕ)
    rot_localfun(x,y,block) = local_align(x,y,block,tivpσ,tivpϕ)
    tiv_branchbound(gmmx, gmmy, tivgmm(gmmx, cx), tivgmm(gmmy, cy); boundsfun=boundsfun, rot_boundsfun=rot_boundsfun, localfun=localfun, rot_localfun=rot_localfun, kwargs...)
end